// Maciej Stefaczyk
// gr. J1I2

#include <string>
#include <iostream>

using namespace std;

// wypisanie informacji o podanym napisie
void info(const string & str, const string & desc) {
	cout << desc << endl;
	cout << "\"" << str << "\"\n";

	// dugo napisu (ilo znakw)
	cout << "length:   " << str.length() << endl;
	// ilo znakw, ktre moe pomieci napis bez alokacji dodatkowej pamici
	// po jej przekroczeniu automatycznie alokowana jest nowa pami
	// o wielkoci dwukrotnie wiksze ni dotychczasowa
	cout << "capacity: " << str.capacity() << endl;
	cout << endl;
}

int main() {
	{
		// napis "123456789"
		string s1("123456789");

		// prosty sposb na odwrcenie kolejnoci znakw w napisie
		// wykorzystujemy konstruktor przyjmujcy jako parametry dwa iteratory
		// napis powstaje przez skopiowanie znakw poczwszy od rbegin
		// (czyli od ostatniego znaku) a do znaku poprzedzajcego rend
		// (czyli do pierwszego znaku)
		string s2(s1.rbegin(), s1.rend());

		cout << "s1: \"" << s1 << "\"\n";
		cout << "s2: \"" << s2 << "\"\n\n";
	}

	{
		// utowrzenie napisu skadajcego si z 10 znakw x
		string s(13, 'x');
		info(s, "string(13, 'x')");
		cout << "Maksymalna dozwolona dugo napisu: " << s.max_size() << " znakw\n";

		// dopisanie na kocu napisu
		s+="yy";
		info(s, "s+=\"yy\"");

		// zarezerwowanie dodatkowego miejsca w pamici dla napisu
		// przy czstych operacjach na napisach umoliwia zredukowanie iloci
		// automatycznych operacji wyduania napisu podczas dopisywania nowych danych
		// UWAGA: zarezerwowane moe by wicej miejsca ni wymagamy
		s.reserve(50);
		info(s, "s.reserve(50)");

		// jeli chcemy zmniejszy ilo miejsca zajmowanego przez napis w pamici
		// do minimum, moemy wymusi rezerwacj zerowej iloci pamici
		s.reserve(0);
		info(s, "s.reserve(0)");

		// jeli chcemy zmieni dugo napisu, nalezy uy funkcji resize
		// wyduenie napisu do 20 znakw, jako nowe znaki dodane zostan mylniki
		s.resize(20, '-');
		info(s, "s.resize(20, '-')");

		// skrcenie napisu
		s.resize(5);
		info(s, "s.resize(5)");

		// sprawdzenie, czy napis jest pusty
		if (!s.empty()) {
			// wyczyszczenie zawartoci napisu
			s.clear();
			info(s, "s.clear()");
		}

	}

	{
		string s;
		string k = "kotara";
		size_t found;

		// przypisanie nowej zawartoci do napisu
		s = "J+a+n ma 123\t lata. 123";

		cout << "\"" << s << "\"\n";

		// wyszukujemy pierwsze wystapienie podanego cigu w napisie
		// jeli zostanie znalezione
		// zamieniamy w napisie 3 znaki poczwszy od pozycji, na ktrej
		// zaczyna si znaleziony cig na nowy pierwsze dwa znaki podanego cigu
		while ( (found = s.find("123")) != string::npos )
			s.replace(found, 3, "42086", 2);

		// wyszukujemy dowolny z podanych znakw a nastepnie usuwamy go z napis
		while ( (found = s.find_first_of("+-\t")) != string::npos )
			s.erase(found, 1);

		// wyszukujemy kade wystpienie cigu 'Jan' a nastepnie wstawiamy za nim
		// dodatkowy cig znakw
		// wyszukiwanie za kadym razem zaczynamy od miejsca, w ktrym wstawilimy nowy cig
		found = 0;
		while ( (found = s.find("Jan", found)) != string::npos ) {
			s.insert(found+3, "usz");
			found += 3;
		}

		// skracamy napis do miejsca, gdzie wystpuje ostatni znak przestankowy
		found = s.find_last_of(".,;");
		if (found != string::npos)
			s.resize(found+1);

		// dodajemy na kocu spacj
		s += ' ';

		// dodajemy na kocu napisu fragment skadajcy si z czci aktualnego napisu
		// zaczynajcej si na pierwszym znaku i zawierajcej 10 znakw
		s.append(s.substr(0, 10));

		// zamieniamy ostatnie sowo Janusz na Ala
		found = s.rfind("Janusz");
		if (found != string::npos)
			s.replace(found, 6, "Ala");

		// dodajemy na kocu pierwsze 5 znakw z napisu k
		s.append(k, 0, 5);

		// zamieniamy ostatni znak na kropk
		s[s.size()-1] = '.';

		cout << "\"" << s << "\"\n\n";
	}

	{
		string str1 ("malutkie drzewo");
		string str2 ("ogromne drzewo");

		// porwnanie dwch napisw w caoci
		if (str1.compare(str2) != 0)
			cout << str1 << " to nie " << str2 << "\n";

		// porwnanie czci pierwszego napisu z podanym wzorcem
		// porwnywane jest 6 znakw poczwszy od znaku na pozycji 9
		if (str1.compare(9, 6, "drzewo") == 0)
			cout << str1 << " to drzewo\n";

		// w ten sposb porwnujemy 6 ostatnich znakw z wzorcem
		if (str2.compare(str2.size()-6, 6, "drzewo") == 0)
			cout << "oraz " << str2 << " to te drzewo\n";

		// porwnanie czci napisu 1 z czci napisu 2
		// pierwsze dwie cyfry odnosz si do napisu 1
		// ostatnie dwie cyfry oznaczaj pozycj i dugo podcigu z drugiego napisu
		if (str1.compare(9, 6, str2, 8, 6) == 0)
			cout << "wic mamy dwa drzewa\n\n";
	}


	return 0;
}
